using System;
using System.Collections.Generic;
using Dakota.Data;
using Dakota.Space;
using Dakota.Utils;
using Microsoft.Xna.Framework;
using Nez;
using Random = Nez.Random;

namespace Dakota.Missions
{
    public class SpaceMission : SceneComponent
    {
        protected SpaceScene SpaceScene => (SpaceScene) Scene;
        protected int PlayerHitCount;
        protected Dictionary<AsteroidSize, int> AsteroidsDestroyed;
        public bool ZoneShiftDisabled;
        private readonly Dictionary<string, Rectangle> _boundaries;
        private readonly Dictionary<string, bool> _lastFrame;
        private Vector2? _autopilotDestination;

        protected SpaceMission()
        {
            ZoneShiftDisabled = false;
            AsteroidsDestroyed = new Dictionary<AsteroidSize, int>();
            foreach (AsteroidSize size in Enum.GetValues(typeof(AsteroidSize))) AsteroidsDestroyed[size] = 0;
            _boundaries = new Dictionary<string, Rectangle>();
            _lastFrame = new Dictionary<string, bool>();
        }

        protected void AddBoundary(string name, Rectangle rect)
        {
            _boundaries[name] = rect;
            _lastFrame[name] = false;
        }

        protected void AutopilotTo(Vector2 to)
        {
            _autopilotDestination = to;
            SpaceScene.CenterText.SetText("AUTOPILOT ENGAGED", 3);
        }

        protected void AutopilotReverse(int magnitude)
        {
            var backwards = -Vector2.Normalize(SpaceScene.Player.GetComponent<SpaceObject>().Velocity);
            AutopilotTo(SpaceScene.Player.Position + backwards * magnitude);
        }

        #region Hooks

        // these methods can be overriden on missions to control behavior

        public virtual string GetMissionStatus()
        {
            return "";
        }

        public virtual void OnStart()
        {
        }

        public virtual void OnAsteroidDestroyed(AsteroidSize size)
        {
            AsteroidsDestroyed[size] += 1;
        }

        public virtual void OnPlayerHit()
        {
            PlayerHitCount += 1;
        }

        public virtual void OnEnterRegion(string name)
        {
        }

        public virtual void OnExitRegion(string name)
        {
        }

        #endregion

        public override void Update()
        {
            // check boundaries
            foreach (var (name, rect) in _boundaries)
            {
                var thisFrame = rect.Contains(SpaceScene.Player.Position);
                if (thisFrame && !_lastFrame[name])
                    OnEnterRegion(name);
                if (!thisFrame && _lastFrame[name])
                    OnExitRegion(name);
                _lastFrame[name] = thisFrame;
            }

            // autopilot adjustments
            if (_autopilotDestination != null)
            {
                var dist = SpaceScene.Player.Position - (Vector2) _autopilotDestination;

                if (dist.Length() < 35)
                {
                    _autopilotDestination = null;
                    SpaceScene.CenterText.SetText("AUTOPILOT DISENGAGED", 3);
                    SpaceScene.Player.GetComponent<SpaceObject>().Velocity = Vector2.Zero;
                    SpaceScene.Player.GetComponent<PlayerShip>().Thrust(0);
                }
                else
                {
                    var targetAngle = DakotaUtils.ToAngle(dist);
                    if (dist.Length() > 100)
                        targetAngle += (float) Math.PI;

                    if (SpaceScene.Player.Rotation - targetAngle > 0.1f)
                        SpaceScene.Player.Rotation -= GameConstants.ShipTurnSpeed * Time.DeltaTime;
                    else if (targetAngle - SpaceScene.Player.Rotation > 0.1f)
                        SpaceScene.Player.Rotation += GameConstants.ShipTurnSpeed * Time.DeltaTime;
                    else
                        SpaceScene.Player.GetComponent<PlayerShip>().Thrust(.01f);
                }
            }
        }
    }
}