using System;
using Nez;
using Random = Nez.Random;

namespace Inky
{
    public enum Alignment
    {
        Left,
        Right,
        Center
    }
    
    public class Lanky : RenderableComponent
    {
        public override float Height => BodyHeight + HeadHeight + LegLength;
        public override float Width => BodyWidth;
        
        public readonly int LegLength;
        public readonly int ArmLength;
        public readonly int BodyWidth;
        public readonly int BodyHeight;
        public readonly int HeadWidth;
        public readonly int HeadHeight;
        
        public Alignment LegAlignment;
        public Alignment ArmAlignment;

        private float _headShift;
        public float HeadShift
        {
            get => _headShift;
            set => _headShift = Math.Clamp(value, -1, 1);
        }

        private float _eyeShift;
        public float EyeShift
        {
            get => _eyeShift;
            set => _eyeShift = Math.Clamp(value, -1, 1);
        }

        private float _mouthShift;
        public float MouthShift
        {
            get => _mouthShift;
            set => _mouthShift = Math.Clamp(value, -1, 1);
        }

        private float _mouthWidth;
        public float MouthWidth
        {
            get => _mouthWidth;
            set => _mouthWidth = Math.Clamp(value, 0, 1);
        }

        private float _legWobble;

        public float LegWobble
        {
            get => _legWobble;
            set => _legWobble = Math.Clamp(value, -1, 1);
        }

        public Lanky()
        {
            // readonly
            BodyHeight = 100;
            LegLength = (int)((Random.NextFloat(0.6f) + 0.6f) * BodyHeight);
            ArmLength = (int)((Random.NextFloat(0.3f) + 0.3f) * BodyHeight);
            BodyWidth = (int)((Random.NextFloat(0.5f) + 0.5f) * BodyHeight);
            HeadWidth = 40;
            HeadHeight = 40;
            
            LegAlignment = Alignment.Center;
            ArmAlignment = Alignment.Center;
            MouthWidth = 0.5f;
        }

        public override void Render(Batcher batcher, Camera camera)
        {
            var thickness = 3f;
            // body
            batcher.DrawHollowRect(
                Entity.Position.X - BodyWidth/2f, 
                Entity.Position.Y - BodyHeight/2f,
                BodyWidth, BodyHeight, Color, thickness
                );
            
            // legs
            var legInset = BodyWidth * 0.15f;
            var legBodyY = Entity.Position.Y + BodyHeight / 2f;
            // center positions
            var leftLegX = Entity.Position.X - BodyWidth / 2f + legInset;
            var rightLegX = Entity.Position.X + BodyWidth / 2f - legInset;

            if (LegAlignment == Alignment.Left)
                rightLegX = leftLegX + legInset;
            else if (LegAlignment == Alignment.Right)
                leftLegX = rightLegX - legInset;

            var leftWobble = LegWobble * LegLength * 0.1f;
            var rightWobble = -LegWobble * LegLength * 0.1f;

            batcher.DrawLine(leftLegX, legBodyY, leftLegX, legBodyY + LegLength + leftWobble, Color);
            batcher.DrawLine(rightLegX, legBodyY, rightLegX, legBodyY + LegLength + rightWobble, Color);
            
            // arms
            var armShift = 10;
            var armY = Entity.Position.Y - BodyHeight / 2f + armShift;
            float leftArmX = 0;
            float rightArmX = 0;
            switch (ArmAlignment)
            {
                case Alignment.Left:
                    leftArmX = Entity.Position.X - BodyWidth / 2f + armShift;
                    rightArmX = leftArmX + armShift;
                    break;
                case Alignment.Right:
                    leftArmX = Entity.Position.X + BodyWidth / 2f - armShift * 2;
                    rightArmX = leftArmX + armShift;
                    break;
                case Alignment.Center:
                    leftArmX = Entity.Position.X - armShift * 2;
                    rightArmX = Entity.Position.X + armShift * 2;
                    break;
            }
            batcher.DrawLine(leftArmX, armY, leftArmX, armY + ArmLength, Color);
            batcher.DrawLine(rightArmX, armY, rightArmX, armY + ArmLength, Color);
            
            // head
            var headX = Entity.Position.X + (BodyWidth / 2f) * HeadShift - HeadWidth/2f;
            var headY = Entity.Position.Y - BodyHeight / 2f - HeadHeight;
            batcher.DrawHollowRect(
                headX,
                Entity.Position.Y - BodyHeight/2f - HeadHeight,
                HeadWidth, HeadHeight, Color, thickness
                );
            var eyeInset = EyeShift * (HeadWidth * 0.3f);
            var eyeSpace = HeadWidth * 0.15f;
            var eyeWidth = 2;
            batcher.DrawHollowRect(headX + HeadWidth/2f + eyeInset - eyeSpace - eyeWidth, headY + eyeSpace, eyeWidth, eyeWidth, Color, thickness);
            batcher.DrawHollowRect(headX + HeadWidth/2f + eyeInset + eyeSpace - eyeWidth, headY + eyeSpace, eyeWidth, eyeWidth, Color, thickness);

            var mouthCenterX = HeadWidth / 2f + MouthShift * HeadWidth / 2f;
            var halfMouth = (HeadWidth * MouthWidth) / 2f;
            batcher.DrawLine(
                Math.Max(headX + mouthCenterX - halfMouth, headX),
                headY + HeadHeight * 0.8f ,
                Math.Min(headX + mouthCenterX + halfMouth, headX + HeadWidth),
                headY + HeadHeight * 0.8f ,
                Color
            );
        }
    }
}