using Lighthouse.Data;
using Microsoft.Xna.Framework;
using Nez;

namespace Lighthouse
{
    public enum SceneState
    {
        World,
        Sky,
        Inventory,
    }
    
    public class MainScene : Scene
    {
        private const int PlayerSpawnX = 400;
        private const int PlayerSpawnY = 200;
        
        private SceneState _state;
        public World World;
        private Dialog _dialog;
        private readonly Entity _playerEntity;
        private readonly Entity _sky;
        private readonly Entity _map;

        public Day CurrentDay
        {
            get
            {
                if (!Days.AllDays.ContainsKey(World.Day))
                    return new Day(World.Day);
                return Days.AllDays[World.Day];
            }
        }

        public MainScene()
        {
            SetDesignResolution(640, 360, SceneResolutionPolicy.ShowAllPixelPerfect);
            Screen.SetSize(1280, 720);
            Locations.Initialize();

            World = new World();
            World.Location = Locations.Island;
            World.Player = new Player(World);
            World.Player.X = PlayerSpawnX;
            World.Player.Y = PlayerSpawnY;
            AddSceneComponent(World);

            const int MapPlayerOffsetX = 19;
            const int MapPlayerOffsetY = 10;

            _map = CreateEntity("map");
            _map.AddComponent(new MapRenderer(World, MapPlayerOffsetX, MapPlayerOffsetY));
            _map.AddComponent(new TimeOfDayRenderer(World));
            var minimap = _map.AddComponent(new Minimap(World));

            _sky = CreateEntity("sky");
            _sky.AddComponent(new SkyRenderer(World));

            var playerPos = new Vector2(16 * MapPlayerOffsetX + 8, 16 * MapPlayerOffsetY + 8);
            _playerEntity = CreateEntity("player", playerPos);
            _playerEntity.AddComponent(World.Player);
            World.Player.Animator = _playerEntity.AddComponent(new PlayerAnimator());

            var dialog = CreateEntity("dialog", playerPos + new Vector2(0, -20));
            _dialog = new Dialog();
            dialog.AddComponent(_dialog);

            var debug = CreateEntity("debug");
            var overlay = new DebugOverlay();
            debug.AddComponent(new DebugOverlay());

            SetState(SceneState.World);
        }

        public void SetState(SceneState state)
        {
            _state = state;
            switch (state)
            {
                case SceneState.World:
                    _map.Enabled = true;
                    _sky.Enabled = false;
                    _playerEntity.Enabled = true;
                    break;
                case SceneState.Sky:
                    _map.Enabled = false;
                    _sky.Enabled = true;
                    _playerEntity.Enabled = false;
                    break;
            }
        }

        public override void Update()
        {
            base.Update();
            
            if (VirtualInput.LookUp.IsPressed && World.IsIndoors)
            {
                SetState(_state == SceneState.World ? SceneState.Sky : SceneState.World);
            }

            for (var index = 0; index < CurrentDay.TimedDialog.Count; index++)
            {
                var e = CurrentDay.TimedDialog[index];
                if (World.Hour >= e.Time && !e.Fired)
                {
                    _dialog.AddThought(e.Thought);
                    e.Fired = true;
                }
            }
        }
    }
}